iQuickUI.define("moduleMessage", ["baseCore"], function(baseCore){
	var divBox = '';
	var isShow = false;
	var hasMessage = false;
	var a_options = {
		// 是否显示关闭图标，默认为true
		isCloseButton: true,
		
		// 是否在初始化时显示消息框，默认为false
		show: false,
		
		// 显示秒数，如果为0，表示需要用户自己关闭
		showSeconds: 0
	};
	
	/**
	 * 
	 * @param {Object} messageContext 消息框内容
	 * @param {Object} messageButtons 消息框按钮，数组类型
	 * @param {Object} callbacks 消息框按钮回调方法，数组类型，与消息框按钮对应
	 * @param {Object} options 配置属性
	 */
	var popMessage = function(messageContext, messageButtons, callbacks, options){
		
		var options = options || {};
		
		for(option in a_options){
			if(options[option] == undefined){
				options[option] = a_options[option];
			}
		}
		
		if(messageContext == undefined || typeof(messageContext) != 'string' || messageContext.length == 0){
			return;
		}
		
		if(hasMessage && !isShow){
			// 如果hasMessage为true，说明已执行成功过一次popMessage，但是没有显示消息框
			divBox = "";
			hasMessage = false;
		}else if(isShow){
			// 如果有消息框显示，需要等待消息框被用户删除或者程序自动删除后才可以激活
			return;
		}
		
		divBox += '<div class="cd-popup-container"><p id="p_messageContext">' + messageContext +'</p>';
		if(messageButtons != undefined && messageButtons.length > 0 && messageButtons.length < 3){
			divBox += '<ul class="cd-buttons" id="p_messageButtons">';
			for(var i = 0; i < messageButtons.length; i++){
				if(messageButtons.length == 1){
					divBox += '<li style="width:'+parseInt(100 / messageButtons.length)+'%;"><a id="p_button_'+i+'" href="javascript:;" style="background: #fc7169;border-radius: 0 0 .25em .25em;">'+messageButtons[i]+'</a></li>';	
				}else{
					divBox += '<li style="width:'+parseInt(100 / messageButtons.length)+'%;"><a id="p_button_'+i+'" href="javascript:;">'+messageButtons[i]+'</a></li>';
				}
				
			}
			divBox += '</ul>';
		}
		
		// 是否增加关闭按钮
		if(options.isCloseButton){
			divBox += '<a id="p_messageClose" href="javascript:;" class="cd-popup-close img-replace">关闭</a>';
		}
		divBox += '</div>';
		
		var pMessageOld = document.getElementById("p_message");
		if(pMessageOld != undefined){
			document.getElementsByTagName('body')[0].removeChild(pMessageOld);
		}
		
		var pMessage = document.createElement("div");
		pMessage.id = "p_message";
		pMessage.className = ["cd-popup"];
		pMessage.setAttribute("role", "alert");
		pMessage.innerHTML = divBox;

		var body = document.getElementsByTagName("body")[0];
		body.appendChild(pMessage);
		
		pMessageClose = document.getElementById("p_messageClose");
		
		// 处理关闭事件
		if(options.isCloseButton){
			pMessageClose.addEventListener('click', function(){
				moduleMessage('hide');
			});
		}
		
		// 处理回调方法
		if (messageButtons != undefined) {
			for (var i = 0; i < messageButtons.length; i++) {
				var pFun = function(i){
					callbacks[i]()
				};
				var messageButton = document.getElementById("p_button_" + i);
				messageButton.onclick = functionCall(pFun, i, messageButton);
			}
		}
		
		hasMessage = true;
		
		// 是否初始化时立刻显示，如果有时间控制，也会立刻显示
		if(options.show || options.showSeconds > 0){
			if(options.showSeconds > 0 && options.showSeconds < 100){
				setTimeout(function(){
					moduleMessage('hide');
				}, parseInt(options.showSeconds + "000"))
			}
			moduleMessage('show');
		}
		
		return this;
	}
	
	// 闭包回调函数，解决循环添加事件
	var functionCall = function(fn, params, obj){
		return function(){
			fn.call(obj || window,params);
		}
	}
	
	// 消息框手动控制方法，可以show, hide, toggle消息框
	var moduleMessage = function(flag){
		switch(flag){
			case 'show':
				if(hasMessage && !isShow){
					var pMessage = document.getElementById("p_message");
					if(pMessage != undefined && typeof(pMessage) == 'object'){
						addClass.call(pMessage, 'is-visible');
						isShow = true;
					}
				}
				break;
			
			case 'hide':
				if(hasMessage && isShow){
					var pMessage = document.getElementById("p_message");
					if(pMessage != undefined && typeof(pMessage) == 'object'){
						removeClass.call(pMessage, 'is-visible');
						isShow = false;
					}
				}
				break;
				
			case 'toggle':
				if(hasMessage && !isShow){
					moduleMessage('show');
				}else if(hasMessage && isShow){
					moduleMessage('hide');
				}
				
				break;
		}
	}
	
	// 获取指定元素属性
	var getAttr = function(attr){
		return this.getAttribute(attr);
	};
	
	// 设置指定元素属性
	var setAttr = function(attr, val){
		this.setAttribute(attr, val);
		return this;
	};
	
	// 编辑元素class
	var editClass = function(mode, data){
		var cls = getAttr.call(this, "class") || '';
		var attr = cls.split(/\s+/);
		switch(mode){
			case 'add':
				return setAttr.call(this, 'class', cls + ' ' + data);
			case 'remove':
				for(var i = 0; i < attr.length; i++){
					if(attr[i] == data){
						attr.splice(i, 1);
					}
				};
				var cls = attr.join(" ");
				cls = cls.replace( /^\s|\s$/g, "" );
				return setAttr.call(this, 'class', cls);
			default:
				return this;
		}
	}
	
	// 增加Class
	var addClass = function(cls){
		return editClass.call(this, 'add', cls);
	}
	
	// 删除class
	var removeClass = function(cls){
		return editClass.call(this, 'remove', cls);
	}
	
	
	/**
	 *  类似网页alert确认框信息，之后一个确认按钮，提醒用户一些信息
	 *  ios和android统一
	 * @param {Object} title 窗口标题
	 * @param {Object} content 窗口类容
	 * @param {Object} btnLable 按钮文字
	 */
	var iAlert = function(title,content,btnLable){
	
	}
	
	
	/**
	 *  定义消息框，该消息框可以是模态与非模态，当用户在消息框之外的地方点击时，自动关闭该窗口
	 *   点击消息框上的按钮后，能通过回调函数来执行对应的方法，回调函数传入对应按钮的索引
	 * @param {Object} title 消息框标题，可以为空，为空即不显示标题栏
	 * @param {Object} content 消息框内容
	 * @param {Object} btnArray 按钮数组
	 * @param {Object} location 消息框对应的位置，默认为center，即中间，其他分别为:top,bottom
	 * @param {Object} isModule 是否为模态窗口, 1为是，0为否
	 * @param {Object} callBack 回调函数，通过捕获用户点击按钮的索引，然后将该索引作为参数传递给该回调函数
	 */
	var iMessagePop = function(title,content,btnArray,location,isModule,callBack){
		
	}
	
	
	return{
		popMessage:popMessage,
		moduleMessage:moduleMessage
	}
});